#ifndef  __STDINC_H
#define   __STDINC_H
/*-----------------------------------------------------------------------------
 *  BHtree : basic class for C++ implementation of BH treecode
 *  S. F. Portegies Zwart 2005
 *-----------------------------------------------------------------------------
 */


#define PR(x)  cerr << #x << " = " << x << " "
#define PRC(x) cerr << #x << " = " << x << ",  "
#define PRL(x) cerr << #x << " = " << x << "\n"

//#include  <stdlib.h>
//#include  <math.h>
//#include  <iostream>

#include  <iostream>
#include  <cmath>                          // to include sqrt(), etc.
#include  <cstdlib>                        // for atoi() and atof()
#include  <unistd.h>                       // for getopt()
//#include  <vector>
#include <assert.h>
#define __MINMAX_DEFINED

typedef double real;
using namespace std;

//#include "vec.h"
//#include "nbody_particle.h"



#endif  // __STDINC_H
